local addonName, at = ...
local core = at.core

local main = CreateFrame('Frame', nil, UIParent)
main:SetScript("OnEvent", function(self, event, ...) self[event](self, ...) end)

core:AddCallback('Initialize', nil, function(self, ...)
	local tab = {}

	tab.enabled = true
	tab.barDisplayEnabled = false
	
	tab.defaultEnabled = false
	--tab.defaultTimers = true -- maybe some day
	
	tab.showBorder = true
	tab.showTime = true
	tab.showDefaultTime = false
	tab.BPR = 6
	tab.scale = 1
	tab.width = 17
	tab.height = 17
	tab.anchor = 'BOTTOMLEFT'
	tab.relative = 'TOPLEFT'
	tab.growth = false
	tab.direction = false
	tab.x = 0
	tab.y = 0
	tab.xOffset = BUFF_HORIZ_SPACING
	tab.yOffset = 0
	
	tab.showPlayerBuff = true
	tab.showPlayerDebuff = true
	tab.showPetBuff = true
	tab.showPetDebuff = true
	tab.showFriendlyBuff = false
	tab.showFriendlyDebuff = false
	tab.showHostileBuff = false
	tab.showHostileDebuff = false
	
	tab.filter = {
		--[1459] = true, -- mage: arcane brilliance
		--[21562] = true, -- priest: Power word: fortitude
	}
	
	self.db.aura = tab

	self:AddCallback('VariablesLoaded', 'aura', function(self)
		self.spells = JamPlatesAccessoriesDB.spells or {}
		if not self.db.aura then
			self.db.aura = self:CopyTable(tab)
		else
			self.db.aura.filter = self:CopyTable(JamPlatesAccessoriesDB[JamPlatesAccessoriesCP].aura.filter)
		end
	end)


	tab = {}
	tab.enabled = true
	tab.barDisplayEnabled = false
	
	tab.showBorder = true
	tab.showTime = true
	tab.showDefaultTime = false
	tab.BPR = 6
	tab.scale = 1
	tab.width = 17
	tab.height = 17
	tab.anchor = 'TOPRIGHT'
	tab.relative = 'LEFT'
	tab.growth = false
	tab.direction = true
	tab.x = 0
	tab.y = 0
	tab.xOffset = 0
	tab.yOffset = 14
	
	tab.showPlayerBuff = true
	tab.showPlayerDebuff = true
	tab.showPetBuff = true
	tab.showPetDebuff = true
	tab.showFriendlyBuff = true
	tab.showFriendlyDebuff = true
	tab.showHostileBuff = true
	tab.showHostileDebuff = true
	
	tab.filter = {
		--[1459] = true, -- mage: arcane brilliance
		--[21562] = true, -- priest: Power word: fortitude
	}
	
	self.db.tracker = self:CopyTable(tab)
	
	tab = {}
	tab.anchor = 'RIGHT'
	tab.relative = 'LEFT'
	
	self.db.tracker = self:CopyTable(tab)
	
	self:AddCallback('VariablesLoaded', 'tracker', function(self)
		if not self.db.tracker then
			self.db.tracker = self:CopyTable(tab)
		else
			self.db.tracker.filter = self:CopyTable(JamPlatesAccessoriesDB[JamPlatesAccessoriesCP].tracker.filter)
		end
	end)
end)

core.tracker = {}
core.spells = {}

--upvalue
local select = select
local UnitPlayerControlled = UnitPlayerControlled
local UnitIsDead = UnitIsDead
local UnitAura = UnitAura
local UnitCanAttack = UnitCanAttack
local strfind, ipairs = string.find, ipairs
local rad, ceil = math.rad, math.ceil
local NumberFontNormalYellow = NumberFontNormalYellow
local SystemFont_Shadow_Med1 = SystemFont_Shadow_Med1
local DebuffTypeColor = DebuffTypeColor
DebuffTypeColor.aura = {r = 0, g = 0.65, b = 0}
local GetTime = GetTime
local wipe = table.wipe

--sets or otherwise creates creates tables for use
local Recyctable = {
	auras = {
		player = {buff = {}, debuff = {}},
		pet = {buff = {}, debuff = {}},
		friendly = {buff = {}, debuff = {}},
		hostile = {buff = {}, debuff = {}},
	},
	tracker = {
		player = {buff = {}, debuff = {}},
		pet = {buff = {}, debuff = {}},
		friendly = {buff = {}, debuff = {}},
		hostile = {buff = {}, debuff = {}},
	},
}
local function ReuseTable()
	wipe(Recyctable.auras.player.buff)
	wipe(Recyctable.auras.player.debuff)
	wipe(Recyctable.auras.pet.buff)
	wipe(Recyctable.auras.pet.debuff)
	wipe(Recyctable.auras.friendly.buff)
	wipe(Recyctable.auras.friendly.debuff)
	wipe(Recyctable.auras.hostile.buff)
	wipe(Recyctable.auras.hostile.debuff)
	
	wipe(Recyctable.tracker.player.buff)
	wipe(Recyctable.tracker.player.debuff)
	wipe(Recyctable.tracker.pet.buff)
	wipe(Recyctable.tracker.pet.debuff)
	wipe(Recyctable.tracker.friendly.buff)
	wipe(Recyctable.tracker.friendly.debuff)
	wipe(Recyctable.tracker.hostile.buff)
	wipe(Recyctable.tracker.hostile.debuff)

	return Recyctable
end


local CreateNameplateAuraFrame
local AuraFrame = {}
do
	local prev = anchor
	local function AddSortFrame(self, name)
		local f = CreateFrame('Frame')
		self[name] = f
		f:SetWidth(self:GetWidth())
		f:SetPoint('BOTTOM', prev, 'TOP')
		prev = f
	end
	CreateNameplateAuraFrame = function(nameplate)
		local anchor = CreateFrame('Frame', nil, nameplate)
		anchor:SetWidth(nameplate:GetWidth())
		--anchor:SetHeight(1)
		anchor:SetPoint('BOTTOM', nameplate, 'TOP') -- work out spacing later
		
		AddSortFrame(anchor, 'PlayerBuff')
		AddSortFrame(anchor, 'PetBuff')
		AddSortFrame(anchor, 'FriendlyBuff')
		
		AddSortFrame(anchor, 'PlayerDebuff')
		AddSortFrame(anchor, 'PetDebuff')
		AddSortFrame(anchor, 'FriendlyDebuff')
		
		AddSortFrame(anchor, 'HostileBuff')
		AddSortFrame(anchor, 'HostileDebuff')
		AuraFrame[nameplate] = anchor
	end
end

-- functions
local function Cooldown_OnUpdate(self, elapsed)
	-- make sure it even needs a timer
	if self.time then
		elapsed = self.time - GetTime()

		-- check if time remaining is greater than a hour, a minute or a second;
		-- hide the timer if none of the above
		if elapsed >= 3600 then
			self.timer:SetFormattedText("%d h", ceil(elapsed / 3600))

		elseif elapsed > 60 then
			self.timer:SetFormattedText("%d m", ceil(elapsed / 60))

		elseif elapsed > 0 then
			self.timer:SetFormattedText("%d s", ceil(elapsed))

		else
			self.timer:SetText("")
			self.time = nil
			--self.Parent:Hide()
		end
	end
end
local function Statusbar_OnUpdate(self, elapsed)
	-- make sure it even needs a timer
	if self.time then
		elapsed = self.time - GetTime()
		self:SetValue(elapsed)

		-- check if time remaining is greater than a hour, a minute or a second;
		-- hide the timer if none of the above
		if elapsed >= 3600 then
			self.timer:SetFormattedText("%d h", ceil(elapsed / 3600))

		elseif elapsed > 60 then
			self.timer:SetFormattedText("%d m", ceil(elapsed / 60))

		elseif elapsed > 0 then
			self.timer:SetFormattedText("%d s", ceil(elapsed))

		else
			self.timer:SetText("")
			self.time = nil
			--self:Hide()  -- it'll happen once the aura falls off.
		end
	end
end

-- font objects
local TIMER_FONT = CreateFont(addonName.. "AuraTimerFont_Shadowed")
TIMER_FONT:SetFont([[Fonts\FRIZQT__.TTF]], 8)
TIMER_FONT:SetTextColor(NumberFontNormalYellow:GetTextColor())
TIMER_FONT:SetShadowColor(0, 0, 0, 1)
TIMER_FONT:SetShadowOffset(SystemFont_Shadow_Med1:GetShadowOffset())

local COUNT_FONT = CreateFont(addonName.. "AuraCountFont")
COUNT_FONT:SetFont([[Fonts\FRIZQT__.TTF]], 8)

local function Aura_OnHide(self)
	self.spellID = nil
end

local function SetColours(bar, color)
	bar:SetStatusBarColor(color.r, color.g, color.b)
	bar.edge:SetVertexColor(color.r, color.g, color.b)
	bar.bg:SetVertexColor(color.r, color.g, color.b)
end

local function CreateBar(parent, nameplate, isCC)
	local db = isCC and core.db.tracker or core.db.aura

	local frame = CreateFrame("StatusBar", nil, parent)
	frame.SetColours = SetColours
	frame:SetHeight(14)
	frame:SetPoint('LEFT', parent, 'RIGHT')
	frame:SetStatusBarTexture("Interface\\TARGETINGFRAME\\UI-StatusBar")
	frame:GetStatusBarTexture():SetHorizTile(false)
	frame:GetStatusBarTexture():SetVertTile(false)
	frame:SetScale(db.scale)
	frame:HookScript('OnUpdate', Statusbar_OnUpdate)
	frame:SetPoint('RIGHT', nameplate, 'RIGHT')
	frame:SetAlpha(0.8)
	
	frame.edge = frame:CreateTexture(nil, "BACKGROUND")
	frame.edge:SetTexture("Interface\\BUTTONS\\WHITE8X8")
	frame.edge:SetAlpha(1)
	frame.edge:SetPoint('TOPRIGHT', frame, 'TOPRIGHT')
	frame.edge:SetPoint('BOTTOMRIGHT', frame, 'BOTTOMRIGHT')
	frame.edge:SetWidth(1)


	frame.bg = frame:CreateTexture(nil, "BACKGROUND")
	frame.bg:SetTexture("Interface\\TARGETINGFRAME\\UI-StatusBar")
	frame.bg:SetAlpha(0.10)
	frame.bg:SetAllPoints(true)

	local timer = frame:CreateFontString()
	timer:SetPoint('LEFT', frame, 'LEFT', 5, 0)
	timer:SetFontObject(TIMER_FONT)
	timer:SetJustifyH('LEFT')
	frame.timer = timer

	local name = frame:CreateFontString()
	name:SetPoint('CENTER', frame, 'CENTER', 0, 0)
	name:SetFontObject(TIMER_FONT)
	name:SetJustifyH('CENTER')
	frame.spellName = name

	local count = frame:CreateFontString()
	count:SetPoint('RIGHT', frame, 'RIGHT', -5, 0)
	count:SetFontObject(COUNT_FONT)
	count:SetJustifyH('RIGHT')
	count:Hide()
	frame.count = count
	
	frame:HookScript('OnHide', Aura_OnHide)
	
	return frame
end
local function CreateButton(parent, isCC)
	local db = isCC and core.db.tracker or core.db.aura
	
	local frame = CreateFrame('Frame', nil, parent)
	frame:SetSize(core.db.aura.width, db.height)
	frame:SetScale(db.scale)

	local border = frame:CreateTexture()
	border:SetTexture("Interface\\Buttons\\UI-Debuff-Overlays")
	border:SetTexCoord(0.296875, 0.5703125, 0, 0.515625)
	border:SetSize(db.width +3, db.height +2)
	border:SetPoint('CENTER', frame, 'CENTER')
	frame.border = border

	local texture = frame:CreateTexture()
	texture:SetAllPoints(frame)
	frame.texture = texture

	local timer = frame:CreateFontString()
	timer:SetPoint('TOP', frame, 'BOTTOM', 0 , -1)
	timer:SetFontObject(TIMER_FONT)
	timer:SetJustifyH('CENTER')
	frame.timer = timer

	-- finally
	local cooldown = CreateFrame('Cooldown', nil, frame, "CooldownFrameTemplate")
	cooldown:SetAllPoints(frame)
	cooldown:SetHideCountdownNumbers(not db.showDefaultTime) -- somehow gets deleted every once in a while...  totally not my fault, maybe?
	cooldown.noCooldownCount = true -- CooldownCount ignore thing
	cooldown:SetReverse(true)
	cooldown.timer = timer
	cooldown:HookScript('OnUpdate', Cooldown_OnUpdate)
	cooldown.Parent = frame
	frame.cooldown = cooldown

	local count = frame:CreateFontString()
	count:SetPoint('BOTTOMRIGHT', frame, 'BOTTOMRIGHT')
	count:SetFontObject(COUNT_FONT)
	count:SetJustifyH('RIGHT')
	count:Hide()
	frame.count = count
	
	-- create a bar for "bar mode".  No, they do not serve drinks.....
	frame.bar = CreateBar(frame, parent, isCC)
	
	frame:HookScript('OnHide', Aura_OnHide)

	return frame
end

local function AdjustAuraTraits(frame, spellID, icon, debuffType, duration, expirationTime, count, isCC)
	local db = isCC and core.db.tracker or core.db.aura
	frame.spellID = spellID

	frame.texture:SetTexture(icon)
	
	frame.debuffType = debuffType
	local color = DebuffTypeColor[debuffType or 'none']
	
	frame.border:Hide()
	if db.showBorder then
		frame.border:Show()
		frame.border:SetVertexColor(color.r, color.g, color.b)
	end
	
	local bar = frame.bar
	if count and count > 1 then
		if not db.barDisplayEnabled then
			frame.count:SetText(count)
			frame.count:Show()
			
		else
			bar.count:SetText(count)
			bar.count:Show()
			frame.count:Hide()
			
		end
	else
		frame.count:Hide()
		bar.count:Hide()
	end
	
	if duration and ( duration > 0 ) then
		if not db.barDisplayEnabled then
			frame.cooldown:Show()
			frame.timer:Show()
			if not db.showTime then
				frame.timer:Hide()
			end
			frame.cooldown:SetHideCountdownNumbers(not db.showDefaultTime)

			frame.cooldown.time = expirationTime
			frame.cooldown.duration = duration
			frame.cooldown:SetCooldown(expirationTime -duration, duration)
		else
			frame.timer:Hide()
			bar:Show()
			bar.timer:Show()
			if not db.showTime then
				bar.timer:Hide()
			end

			bar.time = expirationTime
			bar.duration = duration
			bar:SetMinMaxValues(0, duration)
		end

	else
		frame.cooldown:Hide()
		frame.timer:Hide()

		frame.cooldown.time = nil
		frame.cooldown.duration = nil
		
		bar:Hide()
		bar.timer:Hide()
		bar.time = nil
		bar.duration = nil
		bar:SetMinMaxValues(0, 1)
		bar:SetValue(1)

	end
	
	if db.barDisplayEnabled then
		local height = frame.bar:GetHeight()
		frame:SetSize(height, height)
		frame.bar:SetColours(color)
	else
		frame:SetSize(db.height, db.height)
		bar:Hide()
	end
end

local Growth = {
	Row = {
		[false] = {
			'BOTTOMLEFT',
			'TOPLEFT'
		},
		[true] = {
			'TOPLEFT',
			'BOTTOMLEFT',
		},
	},
	Column = {
		[false] = {
			'BOTTOMLEFT',
			'BOTTOMRIGHT'
		},
		[true] = {
			'BOTTOMRIGHT',
			'BOTTOMLEFT',
		},
	},
}
local function AddAura(self, nameplate, frames, spellID, icon, debuffType, duration, expirationTime, count, isCC, index)
	local db = isCC and self.db.tracker or self.db.aura
	if db.barDisplayEnabled and duration == 0 then return index end

	local frame = frames[index]
	if not frame then
		frame = CreateButton(nameplate, isCC)
		self.plates[nameplate][isCC and "tracker" or "auras"][index] = frame
	end
	AdjustAuraTraits(frame, spellID, icon, debuffType, duration, expirationTime, count, isCC)

	local ind = index -1
	frame:ClearAllPoints()
	if not db.enabled then
		frame:Hide()
	else
		if ind ~= 0 then
			if not db.barDisplayEnabled then
				local bpr = db.BPR
				local growth
				if ceil(ind / bpr) ~= ind / bpr then
					growth = db.direction
					frame:SetPoint(Growth.Column[growth][1], frames[ind], Growth.Column[growth][2], db.xOffset  * (growth and 1 or -1), 0)

				else
					growth = db.growth
					--frame:SetPoint('BOTTOMLEFT', buffs[index - self.db.aura.BPR + 1], 'TOPLEFT', 0, 14)
					frame:SetPoint(Growth.Row[growth][1], frames[ind - bpr + 1], Growth.Row[growth][2], 0, db.yOffset * (growth and -1 or 1) + 14)
					
				end
			else
				frame:SetPoint('BOTTOMLEFT', frames[ind], 'TOPLEFT', 0, db.yOffset)
			end

		else
			if not db.barDisplayEnabled then
				frame:SetPoint(db.anchor, nameplate, db.relative, db.x, db.y)
			else
				frame:SetPoint('BOTTOMLEFT', nameplate, 'TOPLEFT', 0, db.y)
			end
		end
	end
		
	frame:Show()
	return index +1
end

local function SetAuras(self, nameplate, auras)
	local frames = self.plates[nameplate].auras
	local tracker = self.plates[nameplate].tracker
	local index = 1
	local tIndex = 1
	for x, filter in pairs(auras) do
		for y, buff in pairs(filter) do
			for spellID, tab in pairs(buff) do
				if not tab.isCC then
					index = AddAura(self, nameplate, frames, spellID, tab.icon, tab.debuffType, tab.duration, tab.expiration, tab.count, false, index)
				else
					tIndex = AddAura(self, nameplate, tracker, spellID, tab.icon, tab.debuffType, tab.duration, tab.expiration, tab.count, true, tIndex)
				end
			end
		end
	end
	for x = index, #frames do
		frames[x]:Hide()
	end
	for x = tIndex, #tracker do
		tracker[x]:Hide()
	end
end

local function ScanAuras(self, nameplate, token)
	local tab = ReuseTable()
	local aura = self.db.aura
	local trac = self.db.tracker
	
	local index = 1
	
	local filter = 'HARMFUL'
	local unitIsAllied = not UnitCanAttack('player', token)
	
	local spellID, name, icon, count, debuffType, duration, expirationTime, caster = true
	while token and spellID do
		name, _, icon, count, debuffType, duration, expirationTime, caster, _, _, spellID = UnitAura(token, index, filter)
		
		if spellID then
			local tmp = self.spells[spellID] or {}
			tmp.name = name
			tmp.icon = icon
			tmp.debuffType = debuffType
			self.spells[spellID] = tmp
			
			--use it again
			--tmp.spellID = spellID
			tmp.duration = duration
			tmp.expiration = expirationTime
			tmp.count = count
			tmp.isCC = trac.filter[spellID] and trac.enabled
			
			-- neatly organize the auras into categories
			if caster == 'player' or caster == 'pet' then
				if (aura.showPlayerBuff and not filter) then
					tmp.debuffType = 'aura'
					tab.auras.player.buff[spellID] = tmp
				elseif (aura.showPlayerDebuff and filter) then
					tab.auras.player.debuff[spellID] = tmp
				elseif (aura.showPetBuff and not filter) then
					tmp.debuffType = 'aura'
					tab.auras.pet.buff[spellID] = tmp
				elseif (aura.showPetDebuff and filter) then
					tab.auras.pet.debuff[spellID] = tmp
				end
			elseif unitIsAllied then
				if (aura.showFriendlyBuff and not filter) then
					tmp.debuffType = 'aura'
					tab.auras.friendly.buff[spellID] = tmp
				elseif (aura.showFriendlyDebuff and filter) then
					tab.auras.friendly.debuff[spellID] = tmp
				end
			else
				if (aura.showHostileBuff and not filter) then
					tab.auras.hostile.buff[spellID] = tmp
				elseif (aura.showHostileDebuff and filter) then
					tab.auras.hostile.debuff[spellID] = tmp
				end
			end
		end
		
		if filter and not spellID then
			spellID = true
			index = 1
			filter = nil
		else
			index = index + 1
		end

	end
	
	SetAuras(self, nameplate, tab.auras)
end


local UnitAurasEventFrames = {}
core:AddCallback('NAME_PLATE_CREATED', 'aura', function(self, plate)
	self.plates[plate].auras = {}
	self.plates[plate].tracker = {}
	
	CreateNameplateAuraFrame(plate)

	-- in case an addon unregisters the nameplates events.....
	UnitAurasEventFrames[plate] = CreateFrame('Frame', nil, plate)
	UnitAurasEventFrames[plate]:SetScript('OnEvent', function()
		ScanAuras(self, plate, plate.namePlateUnitToken)
	end)
	local visible = self.db.aura.defaultEnabled and main.Show or main.Hide
	if plate.UnitFrame.BuffFrame then visible(plate.UnitFrame.BuffFrame) end
end)

local function ToggleFrames(isCC)
	local trackerBar
	local auraBar
	local trackerIcon
	local auraIcon

	--local db = isCC and core.db.tracker or core.db.aura
	for plate, frame in pairs(core.plates) do
		frame:Hide()
	end
end

local function ToggleEvents(self)
	local aura = self.db.aura.enabled
	local track = self.db.tracker.enabled
	local state = aura or track
	local visible = self.db.aura.defaultEnabled and main.Show or main.Hide
	for x, v in pairs(core.plates) do
		visible(x.UnitFrame.BuffFrame)
	end
	ToggleFrames()
	ToggleFrames(true)
	if state then
		state = 'AddCallback'
	else
		state = 'RemoveCallback'
		for x, v in pairs(UnitAurasEventFrames) do
			v:UnregisterEvent('UNIT_AURA')
		end
	end
	
	self[state](self, 'NAME_PLATE_UNIT_ADDED', 'aura', function(self, plate, token)
		UnitAurasEventFrames[plate]:RegisterUnitEvent('UNIT_AURA', token)
		ScanAuras(self, plate, token)
	end)
	self[state](self, 'NAME_PLATE_UNIT_REMOVED', 'aura', function(self, plate, token)
		UnitAurasEventFrames[plate]:UnregisterEvent('UNIT_AURA')
		local tab = self.plates[plate]
		for x = 1, #tab do
			tab[x]:Hide()
		end
	end)
end


core:AddCallback('Toggle', 'aura', function(self, ...)
	ToggleEvents(self)
end)

core:AddCallback('Toggle', 'tracker', function(self, ...)
	ToggleEvents(self)
end)
